# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------


def cf_synapse_cl(cli_ctx, *_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.synapse import SynapseManagementClient
    return get_mgmt_service_client(cli_ctx,
                                   SynapseManagementClient)


def cf_kusto_operation(cli_ctx, *_):
    return cf_synapse_cl(cli_ctx).kusto_operations


def cf_kusto_pool(cli_ctx, *_):
    return cf_synapse_cl(cli_ctx).kusto_pool


def cf_kusto_pool(cli_ctx, *_):
    return cf_synapse_cl(cli_ctx).kusto_pools


def cf_kusto_pool_attached_database_configuration(cli_ctx, *_):
    return cf_synapse_cl(cli_ctx).kusto_pool_attached_database_configurations


def cf_kusto_pool_database(cli_ctx, *_):
    return cf_synapse_cl(cli_ctx).kusto_pool_databases


def cf_kusto_pool_data_connection(cli_ctx, *_):
    return cf_synapse_cl(cli_ctx).kusto_pool_data_connections


def cf_kusto_pool_principal_assignment(cli_ctx, *_):
    return cf_synapse_cl(cli_ctx).kusto_pool_principal_assignments


def cf_kusto_pool_database_principal_assignment(cli_ctx, *_):
    return cf_synapse_cl(cli_ctx).kusto_pool_database_principal_assignments
